@@ -1,12 +1,14 @@
+from __future__ import annotations
+
+import math
 from collections import deque
 from dataclasses import dataclass
 from types import TracebackType
-from typing import Deque, Optional, Tuple, Type
-from warnings import warn
+
+from sniffio import AsyncLibraryNotFoundError
 
 from ..lowlevel import cancel_shielded_checkpoint, checkpoint, checkpoint_if_cancelled
-from ._compat import DeprecatedAwaitable
-from ._eventloop import get_asynclib
+from ._eventloop import get_async_backend
 from ._exceptions import BusyResourceError, WouldBlock
 from ._tasks import CancelScope
 from ._testing import TaskInfo, get_current_task
@@ -26,15 +28,16 @@ class CapacityLimiterStatistics:
     """
     :ivar int borrowed_tokens: number of tokens currently borrowed by tasks
     :ivar float total_tokens: total number of available tokens
-    :ivar tuple borrowers: tasks or other objects currently holding tokens borrowed from this
-        limiter
-    :ivar int tasks_waiting: number of tasks waiting on :meth:`~.CapacityLimiter.acquire` or
+    :ivar tuple borrowers: tasks or other objects currently holding tokens borrowed from
+        this limiter
+    :ivar int tasks_waiting: number of tasks waiting on
+        :meth:`~.CapacityLimiter.acquire` or
         :meth:`~.CapacityLimiter.acquire_on_behalf_of`
     """
 
     borrowed_tokens: int
     total_tokens: float
-    borrowers: Tuple[object, ...]
+    borrowers: tuple[object, ...]
     tasks_waiting: int
 
 
@@ -42,13 +45,13 @@ class CapacityLimiterStatistics:
 class LockStatistics:
     """
     :ivar bool locked: flag indicating if this lock is locked or not
-    :ivar ~anyio.TaskInfo owner: task currently holding the lock (or ``None`` if the lock is not
-        held by any task)
+    :ivar ~anyio.TaskInfo owner: task currently holding the lock (or ``None`` if the
+        lock is not held by any task)
     :ivar int tasks_waiting: number of tasks waiting on :meth:`~.Lock.acquire`
     """
 
     locked: bool
-    owner: Optional[TaskInfo]
+    owner: TaskInfo | None
     tasks_waiting: int
 
 
@@ -56,7 +59,8 @@ class LockStatistics:
 class ConditionStatistics:
     """
     :ivar int tasks_waiting: number of tasks blocked on :meth:`~.Condition.wait`
-    :ivar ~anyio.LockStatistics lock_statistics: statistics of the underlying :class:`~.Lock`
+    :ivar ~anyio.LockStatistics lock_statistics: statistics of the underlying
+        :class:`~.Lock`
     """
 
     tasks_waiting: int
@@ -69,14 +73,18 @@ class SemaphoreStatistics:
     :ivar int tasks_waiting: number of tasks waiting on :meth:`~.Semaphore.acquire`
 
     """
+
     tasks_waiting: int
 
 
 class Event:
-    def __new__(cls) -> 'Event':
-        return get_asynclib().Event()
+    def __new__(cls) -> Event:
+        try:
+            return get_async_backend().create_event()
+        except AsyncLibraryNotFoundError:
+            return EventAdapter()
 
-    def set(self) -> DeprecatedAwaitable:
+    def set(self) -> None:
         """Set the flag, notifying all listeners."""
         raise NotImplementedError
 
@@ -88,7 +96,8 @@ class Event:
         """
         Wait until the flag has been set.
 
-        If the flag has already been set when this method is called, it returns immediately.
+        If the flag has already been set when this method is called, it returns
+        immediately.
 
         """
         raise NotImplementedError
@@ -98,18 +107,50 @@ class Event:
         raise NotImplementedError
 
 
+class EventAdapter(Event):
+    _internal_event: Event | None = None
+
+    def __new__(cls) -> EventAdapter:
+        return object.__new__(cls)
+
+    @property
+    def _event(self) -> Event:
+        if self._internal_event is None:
+            self._internal_event = get_async_backend().create_event()
+
+        return self._internal_event
+
+    def set(self) -> None:
+        self._event.set()
+
+    def is_set(self) -> bool:
+        return self._internal_event is not None and self._internal_event.is_set()
+
+    async def wait(self) -> None:
+        await self._event.wait()
+
+    def statistics(self) -> EventStatistics:
+        if self._internal_event is None:
+            return EventStatistics(tasks_waiting=0)
+
+        return self._internal_event.statistics()
+
+
 class Lock:
-    _owner_task: Optional[TaskInfo] = None
+    _owner_task: TaskInfo | None = None
 
     def __init__(self) -> None:
-        self._waiters: Deque[Tuple[TaskInfo, Event]] = deque()
+        self._waiters: deque[tuple[TaskInfo, Event]] = deque()
 
     async def __aenter__(self) -> None:
         await self.acquire()
 
-    async def __aexit__(self, exc_type: Optional[Type[BaseException]],
-                        exc_val: Optional[BaseException],
-                        exc_tb: Optional[TracebackType]) -> None:
+    async def __aexit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> None:
         self.release()
 
     async def acquire(self) -> None:
@@ -127,33 +168,39 @@ class Lock:
             except BaseException:
                 if not event.is_set():
                     self._waiters.remove(token)
+                elif self._owner_task == task:
+                    self.release()
 
                 raise
 
             assert self._owner_task == task
         else:
-            await cancel_shielded_checkpoint()
+            try:
+                await cancel_shielded_checkpoint()
+            except BaseException:
+                self.release()
+                raise
 
     def acquire_nowait(self) -> None:
         """
         Acquire the lock, without blocking.
 
-        :raises ~WouldBlock: if the operation would block
+        :raises ~anyio.WouldBlock: if the operation would block
 
         """
         task = get_current_task()
         if self._owner_task == task:
-            raise RuntimeError('Attempted to acquire an already held Lock')
+            raise RuntimeError("Attempted to acquire an already held Lock")
 
         if self._owner_task is not None:
             raise WouldBlock
 
         self._owner_task = task
 
-    def release(self) -> DeprecatedAwaitable:
+    def release(self) -> None:
         """Release the lock."""
         if self._owner_task != get_current_task():
-            raise RuntimeError('The current task is not holding this lock')
+            raise RuntimeError("The current task is not holding this lock")
 
         if self._waiters:
             self._owner_task, event = self._waiters.popleft()
@@ -161,8 +208,6 @@ class Lock:
         else:
             del self._owner_task
 
-        return DeprecatedAwaitable(self.release)
-
     def locked(self) -> bool:
         """Return True if the lock is currently held."""
         return self._owner_task is not None
@@ -177,23 +222,26 @@ class Lock:
 
 
 class Condition:
-    _owner_task: Optional[TaskInfo] = None
+    _owner_task: TaskInfo | None = None
 
-    def __init__(self, lock: Optional[Lock] = None):
+    def __init__(self, lock: Lock | None = None):
         self._lock = lock or Lock()
-        self._waiters: Deque[Event] = deque()
+        self._waiters: deque[Event] = deque()
 
     async def __aenter__(self) -> None:
         await self.acquire()
 
-    async def __aexit__(self, exc_type: Optional[Type[BaseException]],
-                        exc_val: Optional[BaseException],
-                        exc_tb: Optional[TracebackType]) -> None:
+    async def __aexit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> None:
         self.release()
 
     def _check_acquired(self) -> None:
         if self._owner_task != get_current_task():
-            raise RuntimeError('The current task is not holding the underlying lock')
+            raise RuntimeError("The current task is not holding the underlying lock")
 
     async def acquire(self) -> None:
         """Acquire the underlying lock."""
@@ -204,16 +252,15 @@ class Condition:
         """
         Acquire the underlying lock, without blocking.
 
-        :raises ~WouldBlock: if the operation would block
+        :raises ~anyio.WouldBlock: if the operation would block
 
         """
         self._lock.acquire_nowait()
         self._owner_task = get_current_task()
 
-    def release(self) -> DeprecatedAwaitable:
+    def release(self) -> None:
         """Release the underlying lock."""
         self._lock.release()
-        return DeprecatedAwaitable(self.release)
 
     def locked(self) -> bool:
         """Return True if the lock is set."""
@@ -265,28 +312,33 @@ class Condition:
 
 
 class Semaphore:
-    def __init__(self, initial_value: int, *, max_value: Optional[int] = None):
+    def __init__(self, initial_value: int, *, max_value: int | None = None):
         if not isinstance(initial_value, int):
-            raise TypeError('initial_value must be an integer')
+            raise TypeError("initial_value must be an integer")
         if initial_value < 0:
-            raise ValueError('initial_value must be >= 0')
+            raise ValueError("initial_value must be >= 0")
         if max_value is not None:
             if not isinstance(max_value, int):
-                raise TypeError('max_value must be an integer or None')
+                raise TypeError("max_value must be an integer or None")
             if max_value < initial_value:
-                raise ValueError('max_value must be equal to or higher than initial_value')
+                raise ValueError(
+                    "max_value must be equal to or higher than initial_value"
+                )
 
         self._value = initial_value
         self._max_value = max_value
-        self._waiters: Deque[Event] = deque()
+        self._waiters: deque[Event] = deque()
 
-    async def __aenter__(self) -> 'Semaphore':
+    async def __aenter__(self) -> Semaphore:
         await self.acquire()
         return self
 
-    async def __aexit__(self, exc_type: Optional[Type[BaseException]],
-                        exc_val: Optional[BaseException],
-                        exc_tb: Optional[TracebackType]) -> None:
+    async def __aexit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> None:
         self.release()
 
     async def acquire(self) -> None:
@@ -302,16 +354,22 @@ class Semaphore:
             except BaseException:
                 if not event.is_set():
                     self._waiters.remove(event)
+                else:
+                    self.release()
 
                 raise
         else:
-            await cancel_shielded_checkpoint()
+            try:
+                await cancel_shielded_checkpoint()
+            except BaseException:
+                self.release()
+                raise
 
     def acquire_nowait(self) -> None:
         """
         Acquire the underlying lock, without blocking.
 
-        :raises ~WouldBlock: if the operation would block
+        :raises ~anyio.WouldBlock: if the operation would block
 
         """
         if self._value == 0:
@@ -319,25 +377,23 @@ class Semaphore:
 
         self._value -= 1
 
-    def release(self) -> DeprecatedAwaitable:
+    def release(self) -> None:
         """Increment the semaphore value."""
         if self._max_value is not None and self._value == self._max_value:
-            raise ValueError('semaphore released too many times')
+            raise ValueError("semaphore released too many times")
 
         if self._waiters:
             self._waiters.popleft().set()
         else:
             self._value += 1
 
-        return DeprecatedAwaitable(self.release)
-
     @property
     def value(self) -> int:
         """The current value of the semaphore."""
         return self._value
 
     @property
-    def max_value(self) -> Optional[int]:
+    def max_value(self) -> int | None:
         """The maximum value of the semaphore."""
         return self._max_value
 
@@ -351,15 +407,21 @@ class Semaphore:
 
 
 class CapacityLimiter:
-    def __new__(cls, total_tokens: float) -> 'CapacityLimiter':
-        return get_asynclib().CapacityLimiter(total_tokens)
+    def __new__(cls, total_tokens: float) -> CapacityLimiter:
+        try:
+            return get_async_backend().create_capacity_limiter(total_tokens)
+        except AsyncLibraryNotFoundError:
+            return CapacityLimiterAdapter(total_tokens)
 
     async def __aenter__(self) -> None:
         raise NotImplementedError
 
-    async def __aexit__(self, exc_type: Optional[Type[BaseException]],
-                        exc_val: Optional[BaseException],
-                        exc_tb: Optional[TracebackType]) -> Optional[bool]:
+    async def __aexit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> bool | None:
         raise NotImplementedError
 
     @property
@@ -368,7 +430,8 @@ class CapacityLimiter:
         The total number of tokens available for borrowing.
 
         This is a read-write property. If the total number of tokens is increased, the
-        proportionate number of tasks waiting on this limiter will be granted their tokens.
+        proportionate number of tasks waiting on this limiter will be granted their
+        tokens.
 
         .. versionchanged:: 3.0
             The property is now writable.
@@ -380,11 +443,6 @@ class CapacityLimiter:
     def total_tokens(self, value: float) -> None:
         raise NotImplementedError
 
-    async def set_total_tokens(self, value: float) -> None:
-        warn('CapacityLimiter.set_total_tokens has been deprecated. Set the value of the'
-             '"total_tokens" attribute directly.', DeprecationWarning)
-        self.total_tokens = value
-
     @property
     def borrowed_tokens(self) -> int:
         """The number of tokens that have currently been borrowed."""
@@ -395,16 +453,17 @@ class CapacityLimiter:
         """The number of tokens currently available to be borrowed"""
         raise NotImplementedError
 
-    def acquire_nowait(self) -> DeprecatedAwaitable:
+    def acquire_nowait(self) -> None:
         """
-        Acquire a token for the current task without waiting for one to become available.
+        Acquire a token for the current task without waiting for one to become
+        available.
 
         :raises ~anyio.WouldBlock: if there are no tokens available for borrowing
 
         """
         raise NotImplementedError
 
-    def acquire_on_behalf_of_nowait(self, borrower: object) -> DeprecatedAwaitable:
+    def acquire_on_behalf_of_nowait(self, borrower: object) -> None:
         """
         Acquire a token without waiting for one to become available.
 
@@ -416,7 +475,8 @@ class CapacityLimiter:
 
     async def acquire(self) -> None:
         """
-        Acquire a token for the current task, waiting if necessary for one to become available.
+        Acquire a token for the current task, waiting if necessary for one to become
+        available.
 
         """
         raise NotImplementedError
@@ -433,7 +493,9 @@ class CapacityLimiter:
     def release(self) -> None:
         """
         Release the token held by the current task.
-        :raises RuntimeError: if the current task has not borrowed a token from this limiter.
+
+        :raises RuntimeError: if the current task has not borrowed a token from this
+            limiter.
 
         """
         raise NotImplementedError
@@ -442,7 +504,8 @@ class CapacityLimiter:
         """
         Release the token held by the given borrower.
 
-        :raises RuntimeError: if the borrower has not borrowed a token from this limiter.
+        :raises RuntimeError: if the borrower has not borrowed a token from this
+            limiter.
 
         """
         raise NotImplementedError
@@ -457,88 +520,115 @@ class CapacityLimiter:
         raise NotImplementedError
 
 
-def create_lock() -> Lock:
-    """
-    Create an asynchronous lock.
+class CapacityLimiterAdapter(CapacityLimiter):
+    _internal_limiter: CapacityLimiter | None = None
 
-    :return: a lock object
+    def __new__(cls, total_tokens: float) -> CapacityLimiterAdapter:
+        return object.__new__(cls)
 
-    .. deprecated:: 3.0
-       Use :class:`~Lock` directly.
+    def __init__(self, total_tokens: float) -> None:
+        self.total_tokens = total_tokens
 
-    """
-    warn('create_lock() is deprecated -- use Lock() directly', DeprecationWarning)
-    return Lock()
+    @property
+    def _limiter(self) -> CapacityLimiter:
+        if self._internal_limiter is None:
+            self._internal_limiter = get_async_backend().create_capacity_limiter(
+                self._total_tokens
+            )
 
+        return self._internal_limiter
 
-def create_condition(lock: Optional[Lock] = None) -> Condition:
-    """
-    Create an asynchronous condition.
+    async def __aenter__(self) -> None:
+        await self._limiter.__aenter__()
 
-    :param lock: the lock to base the condition object on
-    :return: a condition object
+    async def __aexit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> bool | None:
+        return await self._limiter.__aexit__(exc_type, exc_val, exc_tb)
 
-    .. deprecated:: 3.0
-       Use :class:`~Condition` directly.
+    @property
+    def total_tokens(self) -> float:
+        if self._internal_limiter is None:
+            return self._total_tokens
 
-    """
-    warn('create_condition() is deprecated -- use Condition() directly', DeprecationWarning)
-    return Condition(lock=lock)
+        return self._internal_limiter.total_tokens
 
+    @total_tokens.setter
+    def total_tokens(self, value: float) -> None:
+        if not isinstance(value, int) and value is not math.inf:
+            raise TypeError("total_tokens must be an int or math.inf")
+        elif value < 1:
+            raise ValueError("total_tokens must be >= 1")
 
-def create_event() -> Event:
-    """
-    Create an asynchronous event object.
+        if self._internal_limiter is None:
+            self._total_tokens = value
+            return
 
-    :return: an event object
+        self._limiter.total_tokens = value
 
-    .. deprecated:: 3.0
-       Use :class:`~Event` directly.
+    @property
+    def borrowed_tokens(self) -> int:
+        if self._internal_limiter is None:
+            return 0
 
-    """
-    warn('create_event() is deprecated -- use Event() directly', DeprecationWarning)
-    return get_asynclib().Event()
+        return self._internal_limiter.borrowed_tokens
 
+    @property
+    def available_tokens(self) -> float:
+        if self._internal_limiter is None:
+            return self._total_tokens
 
-def create_semaphore(value: int, *, max_value: Optional[int] = None) -> Semaphore:
-    """
-    Create an asynchronous semaphore.
+        return self._internal_limiter.available_tokens
 
-    :param value: the semaphore's initial value
-    :param max_value: if set, makes this a "bounded" semaphore that raises :exc:`ValueError` if the
-        semaphore's value would exceed this number
-    :return: a semaphore object
+    def acquire_nowait(self) -> None:
+        self._limiter.acquire_nowait()
 
-    .. deprecated:: 3.0
-       Use :class:`~Semaphore` directly.
+    def acquire_on_behalf_of_nowait(self, borrower: object) -> None:
+        self._limiter.acquire_on_behalf_of_nowait(borrower)
 
-    """
-    warn('create_semaphore() is deprecated -- use Semaphore() directly', DeprecationWarning)
-    return Semaphore(value, max_value=max_value)
+    async def acquire(self) -> None:
+        await self._limiter.acquire()
 
+    async def acquire_on_behalf_of(self, borrower: object) -> None:
+        await self._limiter.acquire_on_behalf_of(borrower)
 
-def create_capacity_limiter(total_tokens: float) -> CapacityLimiter:
-    """
-    Create a capacity limiter.
+    def release(self) -> None:
+        self._limiter.release()
 
-    :param total_tokens: the total number of tokens available for borrowing (can be an integer or
-        :data:`math.inf`)
-    :return: a capacity limiter object
+    def release_on_behalf_of(self, borrower: object) -> None:
+        self._limiter.release_on_behalf_of(borrower)
 
-    .. deprecated:: 3.0
-       Use :class:`~CapacityLimiter` directly.
+    def statistics(self) -> CapacityLimiterStatistics:
+        if self._internal_limiter is None:
+            return CapacityLimiterStatistics(
+                borrowed_tokens=0,
+                total_tokens=self.total_tokens,
+                borrowers=(),
+                tasks_waiting=0,
+            )
 
-    """
-    warn('create_capacity_limiter() is deprecated -- use CapacityLimiter() directly',
-         DeprecationWarning)
-    return get_asynclib().CapacityLimiter(total_tokens)
+        return self._internal_limiter.statistics()
 
 
 class ResourceGuard:
-    __slots__ = 'action', '_guarded'
+    """
+    A context manager for ensuring that a resource is only used by a single task at a
+    time.
 
-    def __init__(self, action: str):
-        self.action = action
+    Entering this context manager while the previous has not exited it yet will trigger
+    :exc:`BusyResourceError`.
+
+    :param action: the action to guard against (visible in the :exc:`BusyResourceError`
+        when triggered, e.g. "Another task is already {action} this resource")
+    """
+
+    __slots__ = "action", "_guarded"
+
+    def __init__(self, action: str = "using"):
+        self.action: str = action
         self._guarded = False
 
     def __enter__(self) -> None:
@@ -547,8 +637,11 @@ class ResourceGuard:
 
         self._guarded = True
 
-    def __exit__(self, exc_type: Optional[Type[BaseException]],
-                 exc_val: Optional[BaseException],
-                 exc_tb: Optional[TracebackType]) -> Optional[bool]:
+    def __exit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> bool | None:
         self._guarded = False
         return None
