@@ -1,37 +1,43 @@
+from __future__ import annotations
+
 from abc import abstractmethod
-from typing import Any, Callable, Generic, Optional, TypeVar, Union
+from collections.abc import Callable
+from typing import Any, Generic, TypeVar, Union
 
 from .._core._exceptions import EndOfStream
 from .._core._typedattr import TypedAttributeProvider
 from ._resources import AsyncResource
 from ._tasks import TaskGroup
 
-T_Item = TypeVar('T_Item')
-T_Stream = TypeVar('T_Stream')
+T_Item = TypeVar("T_Item")
+T_co = TypeVar("T_co", covariant=True)
+T_contra = TypeVar("T_contra", contravariant=True)
 
 
-class UnreliableObjectReceiveStream(Generic[T_Item], AsyncResource, TypedAttributeProvider):
+class UnreliableObjectReceiveStream(
+    Generic[T_co], AsyncResource, TypedAttributeProvider
+):
     """
     An interface for receiving objects.
 
-    This interface makes no guarantees that the received messages arrive in the order in which they
-    were sent, or that no messages are missed.
+    This interface makes no guarantees that the received messages arrive in the order in
+    which they were sent, or that no messages are missed.
 
-    Asynchronously iterating over objects of this type will yield objects matching the given type
-    parameter.
+    Asynchronously iterating over objects of this type will yield objects matching the
+    given type parameter.
     """
 
-    def __aiter__(self) -> "UnreliableObjectReceiveStream[T_Item]":
+    def __aiter__(self) -> UnreliableObjectReceiveStream[T_co]:
         return self
 
-    async def __anext__(self) -> T_Item:
+    async def __anext__(self) -> T_co:
         try:
             return await self.receive()
         except EndOfStream:
             raise StopAsyncIteration
 
     @abstractmethod
-    async def receive(self) -> T_Item:
+    async def receive(self) -> T_co:
         """
         Receive the next item.
 
@@ -43,16 +49,18 @@ class UnreliableObjectReceiveStream(Generic[T_Item], AsyncResource, TypedAttribu
         """
 
 
-class UnreliableObjectSendStream(Generic[T_Item], AsyncResource, TypedAttributeProvider):
+class UnreliableObjectSendStream(
+    Generic[T_contra], AsyncResource, TypedAttributeProvider
+):
     """
     An interface for sending objects.
 
-    This interface makes no guarantees that the messages sent will reach the recipient(s) in the
-    same order in which they were sent, or at all.
+    This interface makes no guarantees that the messages sent will reach the
+    recipient(s) in the same order in which they were sent, or at all.
     """
 
     @abstractmethod
-    async def send(self, item: T_Item) -> None:
+    async def send(self, item: T_contra) -> None:
         """
         Send an item to the peer(s).
 
@@ -64,41 +72,46 @@ class UnreliableObjectSendStream(Generic[T_Item], AsyncResource, TypedAttributeP
         """
 
 
-class UnreliableObjectStream(UnreliableObjectReceiveStream[T_Item],
-                             UnreliableObjectSendStream[T_Item]):
+class UnreliableObjectStream(
+    UnreliableObjectReceiveStream[T_Item], UnreliableObjectSendStream[T_Item]
+):
     """
-    A bidirectional message stream which does not guarantee the order or reliability of message
+    A bidirectional message stream which does not guarantee the order or reliability of
+    message delivery.
+    """
+
+
+class ObjectReceiveStream(UnreliableObjectReceiveStream[T_co]):
+    """
+    A receive message stream which guarantees that messages are received in the same
+    order in which they were sent, and that no messages are missed.
+    """
+
+
+class ObjectSendStream(UnreliableObjectSendStream[T_contra]):
+    """
+    A send message stream which guarantees that messages are delivered in the same order
+    in which they were sent, without missing any messages in the middle.
+    """
+
+
+class ObjectStream(
+    ObjectReceiveStream[T_Item],
+    ObjectSendStream[T_Item],
+    UnreliableObjectStream[T_Item],
+):
+    """
+    A bidirectional message stream which guarantees the order and reliability of message
     delivery.
     """
 
-
-class ObjectReceiveStream(UnreliableObjectReceiveStream[T_Item]):
-    """
-    A receive message stream which guarantees that messages are received in the same order in
-    which they were sent, and that no messages are missed.
-    """
-
-
-class ObjectSendStream(UnreliableObjectSendStream[T_Item]):
-    """
-    A send message stream which guarantees that messages are delivered in the same order in which
-    they were sent, without missing any messages in the middle.
-    """
-
-
-class ObjectStream(ObjectReceiveStream[T_Item], ObjectSendStream[T_Item],
-                   UnreliableObjectStream[T_Item]):
-    """
-    A bidirectional message stream which guarantees the order and reliability of message delivery.
-    """
-
     @abstractmethod
     async def send_eof(self) -> None:
         """
         Send an end-of-file indication to the peer.
 
-        You should not try to send any further data to this stream after calling this method.
-        This method is idempotent (does nothing on successive calls).
+        You should not try to send any further data to this stream after calling this
+        method. This method is idempotent (does nothing on successive calls).
         """
 
 
@@ -106,11 +119,11 @@ class ByteReceiveStream(AsyncResource, TypedAttributeProvider):
     """
     An interface for receiving bytes from a single peer.
 
-    Iterating this byte stream will yield a byte string of arbitrary length, but no more than
-    65536 bytes.
+    Iterating this byte stream will yield a byte string of arbitrary length, but no more
+    than 65536 bytes.
     """
 
-    def __aiter__(self) -> 'ByteReceiveStream':
+    def __aiter__(self) -> ByteReceiveStream:
         return self
 
     async def __anext__(self) -> bytes:
@@ -124,8 +137,8 @@ class ByteReceiveStream(AsyncResource, TypedAttributeProvider):
         """
         Receive at most ``max_bytes`` bytes from the peer.
 
-        .. note:: Implementors of this interface should not return an empty :cls:`bytes` object,
-            and users should ignore them.
+        .. note:: Implementors of this interface should not return an empty
+            :class:`bytes` object, and users should ignore them.
 
         :param max_bytes: maximum number of bytes to receive
         :return: the received bytes
@@ -153,13 +166,15 @@ class ByteStream(ByteReceiveStream, ByteSendStream):
         """
         Send an end-of-file indication to the peer.
 
-        You should not try to send any further data to this stream after calling this method.
-        This method is idempotent (does nothing on successive calls).
+        You should not try to send any further data to this stream after calling this
+        method. This method is idempotent (does nothing on successive calls).
         """
 
 
 #: Type alias for all unreliable bytes-oriented receive streams.
-AnyUnreliableByteReceiveStream = Union[UnreliableObjectReceiveStream[bytes], ByteReceiveStream]
+AnyUnreliableByteReceiveStream = Union[
+    UnreliableObjectReceiveStream[bytes], ByteReceiveStream
+]
 #: Type alias for all unreliable bytes-oriented send streams.
 AnyUnreliableByteSendStream = Union[UnreliableObjectSendStream[bytes], ByteSendStream]
 #: Type alias for all unreliable bytes-oriented streams.
@@ -172,16 +187,17 @@ AnyByteSendStream = Union[ObjectSendStream[bytes], ByteSendStream]
 AnyByteStream = Union[ObjectStream[bytes], ByteStream]
 
 
-class Listener(Generic[T_Stream], AsyncResource, TypedAttributeProvider):
+class Listener(Generic[T_co], AsyncResource, TypedAttributeProvider):
     """An interface for objects that let you accept incoming connections."""
 
     @abstractmethod
-    async def serve(self, handler: Callable[[T_Stream], Any],
-                    task_group: Optional[TaskGroup] = None) -> None:
+    async def serve(
+        self, handler: Callable[[T_co], Any], task_group: TaskGroup | None = None
+    ) -> None:
         """
         Accept incoming connections as they come in and start tasks to handle them.
 
         :param handler: a callable that will be used to handle each accepted connection
-        :param task_group: the task group that will be used to start tasks for handling each
-            accepted connection (if omitted, an ad-hoc task group will be created)
+        :param task_group: the task group that will be used to start tasks for handling
+            each accepted connection (if omitted, an ad-hoc task group will be created)
         """
