@@ -1,11 +1,19 @@
+from __future__ import annotations
+
+from collections.abc import Callable, Mapping
 from io import SEEK_SET, UnsupportedOperation
 from os import PathLike
 from pathlib import Path
-from typing import Any, BinaryIO, Callable, Dict, Mapping, Union, cast
+from typing import Any, BinaryIO, cast
 
 from .. import (
-    BrokenResourceError, ClosedResourceError, EndOfStream, TypedAttributeSet, to_thread,
-    typed_attribute)
+    BrokenResourceError,
+    ClosedResourceError,
+    EndOfStream,
+    TypedAttributeSet,
+    to_thread,
+    typed_attribute,
+)
 from ..abc import ByteReceiveStream, ByteSendStream
 
 
@@ -27,11 +35,11 @@ class _BaseFileStream:
 
     @property
     def extra_attributes(self) -> Mapping[Any, Callable[[], Any]]:
-        attributes: Dict[Any, Callable[[], Any]] = {
+        attributes: dict[Any, Callable[[], Any]] = {
             FileStreamAttribute.file: lambda: self._file,
         }
 
-        if hasattr(self._file, 'name'):
+        if hasattr(self._file, "name"):
             attributes[FileStreamAttribute.path] = lambda: Path(self._file.name)
 
         try:
@@ -54,14 +62,14 @@ class FileReadStream(_BaseFileStream, ByteReceiveStream):
     """
 
     @classmethod
-    async def from_path(cls, path: Union[str, PathLike]) -> 'FileReadStream':
+    async def from_path(cls, path: str | PathLike[str]) -> FileReadStream:
         """
         Create a file read stream by opening the given file.
 
         :param path: path of the file to read from
 
         """
-        file = await to_thread.run_sync(Path(path).open, 'rb')
+        file = await to_thread.run_sync(Path(path).open, "rb")
         return cls(cast(BinaryIO, file))
 
     async def receive(self, max_bytes: int = 65536) -> bytes:
@@ -116,17 +124,18 @@ class FileWriteStream(_BaseFileStream, ByteSendStream):
     """
 
     @classmethod
-    async def from_path(cls, path: Union[str, PathLike],
-                        append: bool = False) -> 'FileWriteStream':
+    async def from_path(
+        cls, path: str | PathLike[str], append: bool = False
+    ) -> FileWriteStream:
         """
         Create a file write stream by opening the given file for writing.
 
         :param path: path of the file to write to
-        :param append: if ``True``, open the file for appending; if ``False``, any existing file
-            at the given path will be truncated
+        :param append: if ``True``, open the file for appending; if ``False``, any
+            existing file at the given path will be truncated
 
         """
-        mode = 'ab' if append else 'wb'
+        mode = "ab" if append else "wb"
         file = await to_thread.run_sync(Path(path).open, mode)
         return cls(cast(BinaryIO, file))
 
